@@ -23,8 +23,10 @@ import cl_overriding
 from cl_vars_share import varsShare, clLocale
 from os.path import exists as pathexists
 from os import path
-from cl_utils import isMount, genpassword, getAvailableX11Drivers, \
-                     listDirectory,isPkgInstalled,lspci
+from cl_utils import isMount, genpassword, \
+                     getAvailableVideo, process, \
+                     listDirectory,isPkgInstalled,lspci, readLinesFile, \
+                     getUdevDeviceInfo
 from utils import ip
 from encrypt import getHash
 
@@ -186,22 +188,24 @@ class fillVars(varsShare):
             rootparam=re_res.group(1)
             # check root for /dev/sd view
             if re.match("^\/dev\/[a-z]+.*$", rootparam):
-                return rootparam
+                return getUdevDeviceInfo(
+                   name=rootparam.strip()).get('DEVNAME','')
             # check root set by uuid
-            uuidpath = '/dev/disk/by-uuid'
-            if os.access(uuidpath,os.R_OK):
-                uuidDevs = filter(path.islink,
-                              map(lambda x: path.join(uuidpath,x),
-                                  os.listdir(uuidpath)))
-                mapUuidDev = dict(map(lambda x:(path.basename(x),
-                                path.normpath(path.join(uuidpath,
-                                os.readlink(x)))), uuidDevs))
-            else:
-                mapUuidDev = {}
             if re.match("^UUID=.*$",rootparam):
-                uuid = rootparam[5:]
-                if uuid in mapUuidDev:
-                    return mapUuidDev[uuid]
+                uuid = rootparam[5:].strip("\"'")
+                blkidProcess = process('/sbin/blkid','-c','/dev/null','-U',
+                                    uuid)
+                if blkidProcess.success():
+                    return getUdevDeviceInfo(
+                       name=blkidProcess.read().strip()).get('DEVNAME','')
+            # check root set by label
+            if re.match("^LABEL=.*$",rootparam):
+                uuid = rootparam[6:].strip("\"'")
+                blkidProcess = process('/sbin/blkid','-c','/dev/null','-L',
+                                    uuid)
+                if blkidProcess.success():
+                    return getUdevDeviceInfo(
+                       name=blkidProcess.read().strip()).get('DEVNAME','')
         # get device mounted to root
         dfLines = self._runos("LANG=C df /")
         if not dfLines:
@@ -388,23 +392,25 @@ class fillVars(varsShare):
         xorg_conf = '/etc/X11/xorg.conf'
         # Try analize Xorg.{DISPLAY}.log
         display = os.environ.get('DISPLAY')
-        list_avialable_drivers = \
-            getAvailableX11Drivers(prefix=self.Get('cl_chroot_path'))
-        if display and list_avialable_drivers:
+        list_available_drivers = \
+            getAvailableVideo(prefix=self.Get('cl_chroot_path'))
+        if display and list_available_drivers:
             reDriver = re.compile('|'.join(map(lambda x: "%s_drv.so"%x,
-                                               list_avialable_drivers)))
-            display_number = re.search(r':(\d+)\..*', display)
+                                               list_available_drivers)))
+            display_number = re.search(r':(\d+)(\..*)?', display)
+            reDriverName = re.compile(r'([^/]+)_drv.so')
             if display_number:
                 xorg_log_file = '/var/log/Xorg.%s.log' % \
                                 display_number.group(1)
                 if path.exists(xorg_log_file):
-                    matchStrs = [i for i in open(xorg_log_file)
-                                    if "drv" in i and reDriver.search(i)]
+                    matchStrs = \
+                        map(lambda x:x.group(1),
+                        filter(lambda x:x,
+                        map(reDriverName.search,
+                        filter(lambda x:"drv" in x and reDriver.search(x),
+                        readLinesFile(xorg_log_file)))))
                     if matchStrs:
-                        resDriver = re.search(r'([^/]+)_drv.so',
-                            matchStrs[-1])
-                        if resDriver:
-                            return resDriver.group(1)
+                        return matchStrs[-1]
 
         # analize /etc/X11/xorg.conf
         if path.exists(xorg_conf):
@@ -413,19 +419,15 @@ class fillVars(varsShare):
             if matchSect:
                 resDriver = re.search(r'^\S*Driver\s*"([^"]+)"',
                                       matchSect.group(0),re.S)
-                if resDriver and resDriver.group(1) in list_avialable_drivers:
+                if resDriver and resDriver.group(1) in list_available_drivers:
                     return resDriver.group(1)
 
         videoVal = self.getValueFromCmdLine("calculate","video")
         videoVal = {'i915':'intel'}.get(videoVal,videoVal)
         if not isPkgInstalled('xorg-server') or \
-            videoVal in list_avialable_drivers:
+            videoVal in list_available_drivers:
             return videoVal
-        workedModules = map(lambda x:x[0],
-                        filter(lambda x:x[1].isdigit() and int(x[1])>0,
-                        map(lambda x:x.split()[:3:2],
-                        open('/proc/modules','r'))))
-        return self.getVideoByDefault(list_avialable_drivers)
+        return self.getVideoByDefault(list_available_drivers)
 
     def get_os_x11_height(self):
         """Get screen height in pixeles"""
@@ -559,19 +561,7 @@ class fillVars(varsShare):
 
     def get_cl_kernel_uid(self):
         """Get UID of symlink kernel, initramfs and System.map"""
-        uuidpath = '/dev/disk/by-uuid'
-        if not os.access(uuidpath,os.R_OK):
-            return ""
-        uuidDevs = filter(path.islink,map(lambda x: path.join(uuidpath,x),
-                                          os.listdir(uuidpath)))
-        mapDevUuid = dict(map(lambda x:(path.normpath(path.join(uuidpath,
-                                                      os.readlink(x))),
-                                        path.basename(x)),
-                          uuidDevs))
-        if self.Get('os_root_dev') in mapDevUuid:
-            return mapDevUuid[self.Get('os_root_dev')][:8]
-        else:
-            return ""
+        return self.getKernelUid(self.Get('os_root_dev'))
 
     def get_cl_chroot_status(self):
         """Detect chroot mode by mtab content"""
