@@ -282,7 +282,7 @@ class opt(optparse.OptionParser):
         }]
     variable_view = \
         [{'shortOption':"v",
-         'longOption':"vars",
+         'longOption':"show-variables",
          'action':'count',
          'help':_("print variables, if the two options is verbose "
                   "(added print hidden variables)")
@@ -290,11 +290,11 @@ class opt(optparse.OptionParser):
         {'longOption':"filter",
          'optVal':"FILTER",
          'help':_("filtered variables (as a regular expression is used *), \
-is used in conjunction with the options '-v --vars'")
+is used in conjunction with the options '-v --show-variables'")
         },
         {'longOption':"xml",
          'help':_("output variables in the format of XML, \
-is used in conjunction with the option '-v --vars'")
+is used in conjunction with the option '-v --show-variables'")
         }]
     variable_control = variable_set + variable_view
 
