@@ -16,7 +16,7 @@
 
 import os
 import sys
-from cl_utils import getpathenv, runOsCommand, typeFile
+from cl_utils import getpathenv, runOsCommand, typeFile,process
 import re
 from os import path, R_OK
 try:
@@ -528,6 +528,16 @@ class varsShare:
                 if reRes:
                     return reRes[0]
 
+    def selectVar(self,selField,where="",eq=""):
+        """Select value from matrix variables
+
+        Example:
+        selectVar("os_disk_dev",where="os_disk_mount",eq="/")"""
+        res = filter(lambda x:x[1] == eq,
+              zip(self.Get(selField),
+                  self.Get(where))) or [("","")]
+        return res[0][0]
+
     def getVersionFromGentooFiles(self,systemroot):
         """Get version from gentoo files"""
         gentooFile = path.join(systemroot,"etc/gentoo-release")
@@ -582,16 +592,22 @@ class varsShare:
                         filter(lambda x:x[1].isdigit() and int(x[1])>0,
                         map(lambda x:x.split()[:3:2],
                         open('/proc/modules','r'))))
-        if "nvidia" in drivers_list:
-            defaultNvidia = "nvidia"
-        elif "nouveau" in workedModules:
+        if "nouveau" in workedModules:
             defaultNvidia = "nouveau"
+        elif "nvidia" in drivers_list:
+            defaultNvidia = "nvidia"
         else:
             defaultNvidia = "nv"
+        if "radeon" in workedModules:
+            defaultAti = "radeon"
+        elif "fglrx" in drivers_list:
+            defaultAti = "fglrx"
+        else:
+            defaultAti = "radeon"
         defaultDriver = {
             'vesa':'vesa',
             'nvidia':defaultNvidia,
-            'ati':'fglrx' if "fglrx" in drivers_list else "radeon",
+            'ati':defaultAti,
             'intel':'intel',
             'via':'via',
             'vmware':'vmware'}
@@ -601,3 +617,13 @@ class varsShare:
             return defaultDriver[hr_video]
         else:
             return "vesa"
+
+    def getKernelUid(self,device):
+        """Get Kernel UID by UUID of device"""
+        blkidProcess = process('/sbin/blkid','-c','/dev/null','-s','UUID',
+                                '-o','value',device)
+        res = blkidProcess.read().strip()
+        if res:
+            return res[:8]
+        else:
+            return "no_uid"
